#
# This makefile contains some global definitions,
# that are used during the build process.
# It is included by all the subridrectory libraries.
#


##############################################################
##### 64-BIT CROSS-COMPILATION #####
CXXFLAGS=
FWDPARAM=
ifeq ($(BITS64), true)
  ##### CHANGE WHEN DIFFERENT 64BIT g++ PREFIX ##### 
  #CROSS_COMPILE = x86_64-linux-
  ##### CHANGE WHEN DIFFERENT 64BIT g++ PREFIX ##### 
  CXXFLAGS += -m64
  FWDPARAM += BITS64=true
else
  CXXFLAGS += -m32
endif

# disable cross-compile prefix if CXX not exists
CXX=$(CROSS_COMPILE)g++
CXX2=$(notdir $(shell which $(CXX) 2>/dev/null))
ifneq ("$(CXX)", "$(CXX2)")
  CROSS_COMPILE=
endif

# compilation tools
CC = $(CROSS_COMPILE)g++
CXX = $(CROSS_COMPILE)g++
AR = $(CROSS_COMPILE)ar
RANLIB = $(CROSS_COMPILE)ranlib
AS = $(CROSS_COMPILE)as




##############################################################
##### PATH TO CUDA TOOLKIT #####
#CUDA_TK_BASE=/usr/local/share/cuda-3.2.12
#CUDA_TK_BASE=/usr/local/share/cuda-4.2.9
#CUDA_TK_BASE=/usr/local/share/cuda-5.5.22
CUDA_TK_BASE=/usr/local/share/cuda-6.5.14
##### PATH TO CUDA TOOLKIT #####


##############################################################
##### SELECT BLAS #####
##### options : (ATLAS,GotoBLAS,MKL)
#####
##### ATLAS is the safest option, no possible race conditions 
##### were reported by helgrind. ATLAS might be a bit slower 
##### than the other two BLAS libraries, but is more stable.
ifndef BLAS
  BLAS=ATLAS
endif
FWDPARAM += BLAS=$(BLAS)
ifeq ($(BLAS), GotoBLAS)
  CXXFLAGS += -DADD_CLAPACK_ITF
endif
ifeq ($(BLAS), MKL)
  CXXFLAGS += -DADD_CLAPACK_ITF
endif
#MKL cannot be freely redistributed, so we will source it from path:
MKLROOT=/mnt/matylda5/iveselyk/lib/mkl/
##### SELECT BLAS #####


# compilation args
CXXFLAGS += -g -Wall -O2 -DHAVE_BLAS -rdynamic
CXXFLAGS += -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings 
CXXFLAGS += -Wno-conversion -Wno-sign-compare

# enable double-precision
ifeq ($(DOUBLEPRECISION), true)
  CXXFLAGS += -DDOUBLEPRECISION
  FWDPARAM += DOUBLEPRECISION=true
endif


# compile all the source .cc files 
SRC=$(wildcard *.cc)
OBJ=$(patsubst %.cc, %.o, $(SRC))




#########################################################
# CONFIGURATION CHECKS
#

#check that CUDA_TK_BASE is set correctly
ifeq ("$(wildcard $(CUDA_TK_BASE)/bin/nvcc)", "$(CUDA_TK_BASE)/bin/nvcc")
  HAVE_CUDA=true
else 
  ifeq ($(CUDA), true)
    $(error %%% CUDA not found! Incorrect path in CUDA_TK_BASE: $(CUDA_TK_BASE) in 'trunk/src/tnet.mk')
  endif
endif

#
#########################################################


