
# ./gen_mlp_init.py
# script generateing NN initialization for training with TNet
#     
# author: Karel Vesely
#

import math, random
import sys


from optparse import OptionParser

parser = OptionParser()
parser.add_option('--dim', dest='dim', help='d1:d2:d3 layer dimensions in the network')
parser.add_option('--gauss', dest='gauss', help='use gaussian noise for weights', action='store_true', default=False)
parser.add_option('--gauss-scale', dest='gauss_scale', help='scaling factor for the gaussian noise', default=0.1)
parser.add_option('--negbias', dest='negbias', help='use uniform [-4.1,-3.9] for bias (defaultall 0.0)', action='store_true', default=False)
parser.add_option('--inputscale', dest='inputscale', help='scale the weights by 3/sqrt(Ninputs)', action='store_true', default=False)
parser.add_option('--activation', dest='activation', help='activation type tag (def. <sigmoid>)', default='<sigmoid>')
parser.add_option('--linBNdim', dest='linBNdim', help='dim of linear bottleneck (sigmoids will be omitted, bias will be zero)',default=0)
parser.add_option('--linOutput', dest='linOutput', help='generate MLP with linear output', action='store_true', default=False)
parser.add_option('--linOutput-bias', dest='linOutput_bias', help='use TNorm norm file to initialize the output biases of linear output', default="")
(options, args) = parser.parse_args()

if(options.dim == None):
    parser.print_help()
    sys.exit(1)


dimStrL = options.dim.split(':')

dimL = []
for i in range(len(dimStrL)):
    dimL.append(int(dimStrL[i]))

#get the gauss scale option, we might need it, normally it is 0.1
gauss_scale=float(options.gauss_scale)


#print dimL,'linBN',options.linBNdim

for layer in range(len(dimL)-1):
    print '<biasedlinearity>', dimL[layer+1], dimL[layer]
    #the weight matrix
    print 'm', dimL[layer+1], dimL[layer]
    for row in range(dimL[layer+1]):
        for col in range(dimL[layer]):
            if(options.gauss):
                if(options.inputscale):
                    print 3/math.sqrt(dimL[layer])*random.gauss(0.0,1.0),
                else:
                    print gauss_scale*random.gauss(0.0,1.0),
            else:
                if(options.inputscale):
                    print (random.random()-0.5)*2*3/math.sqrt(dimL[layer]),
                else:
                    print random.random()/5.0-0.1, 
        print
    #the bias vector
    print 'v', dimL[layer+1]
    #take the mean stats as bias vector for MLP with linear output
    if(layer == len(dimL)-2 and options.linOutput and options.linOutput_bias != ""): #output layer of MLP with linear output, use mean values as bias
        f=open(options.linOutput_bias,'r')
        arr = f.readline().split(' ')
        assert(arr[0] == '<bias>')
        assert(int(arr[1]) == dimL[layer+1])
        arr = f.readline().strip().replace('  ',' ').split(' ')
        f.close()
        for i in range(2,len(arr)): #start by position 2, arr[0]=='v' arr[2]==dim
            print -float(arr[i]),
    else:
        #generate the biases
        for idx in range(dimL[layer+1]):
            if(int(options.linBNdim) == dimL[layer+1]): #linear bottleneck -> use zero
                print '0.0',
            elif(layer == len(dimL)-2): #layer before softmax -> use zero
                print '0.0',
            elif(options.negbias): #negative bias -> use U[-4.1,-3.9]
                print random.random()/5.0-4.1,
            else: #otherwise zero bias
                print '0.0',
    print

    #the activation function
    if(int(options.linBNdim) != dimL[layer+1]):
        if(layer == len(dimL)-2):
            if(not(options.linOutput)) :
               print '<softmax>', dimL[layer+1], dimL[layer+1]
        else:
            print options.activation, dimL[layer+1], dimL[layer+1]



