#!/bin/bash

#$ -o _$JOB_NAME.out
#$ -e _$JOB_NAME.err
#$ -cwd

##$ -q long.q@@pco203,long.q@pcspeech-gpu
##$ -l gpu=1

if [ $SGE_O_WORKDIR ];then cd $SGE_O_WORKDIR; fi

#TNET_ROOT='/mnt/matylda5/iveselyk/DEVEL/TNet_PACKAGE/TNet/trunk/'
TNET_ROOT='/mnt/matylda5/iveselyk/Tools/TNet/'
#TNET_ROOT='/mnt/matylda5/iveselyk/Tools/TNet64/'
 
#########################################################
# SETUP
#
#set the input files
SCP_TRAIN='/mnt/matylda5/iveselyk/DATABASE/TIMIT/lists/fbank23_train.scp'
#STK_CONF='norm.conf' # set if we need cmn/cvn of input features (FBANKs)
#MMF='' # set MMF file to bypass generation of transform with Hamming,DCT

#set input transform parameters
FRM_EXT=15  # left/right temporal context in frames, total context is FRM_EXT*2+1
DCT_BASE=16 # number of basis in DCT transform (including C0)
DIM_IN=$(HList -z -h $(head -n 1 $SCP_TRAIN) | grep Comps | awk '{ print $3; }') # dimensionality of raw features

#
#
#########################################################


#Generate the transform, when not supplied by MMF
if [ ! $MMF ]; then
  MMF="tr_${DIM_IN}Tcontext$((2*FRM_EXT + 1))_Ham_dct${DCT_BASE}"
  if [ ! -e $MMF ]; then
    python $TNET_ROOT/tools/transform/gen_hamm_dct.py \
    --dimIn=$DIM_IN \
    --startFrmExt=$FRM_EXT \
    --endFrmExt=$FRM_EXT \
    --dctBaseCnt=$DCT_BASE \
    > $MMF
  fi
fi

#output file with cmn cvn normalization
NORM=${MMF}${STK_CONF:+_$(basename ${STK_CONF} .conf)}.norm

#compute the normalization
$TNET_ROOT/src/TNorm -D -A -T 1 -S $SCP_TRAIN ${STK_CONF:+ -C $STK_CONF} -H $MMF \
  --TARGET-MMF=$NORM \
  --START-FRM-EXT=$FRM_EXT \
  --END-FRM-EXT=$FRM_EXT || { echo "$0 error occured"; exit; }

#assemble the transform from MMF and normalization
cat $MMF $NORM > $(basename $NORM .norm).transf

echo "TNorm finished OK"

